/*  
    Copyright 2007, Francois Schiettekatte

    This file is part of Corteo.

    Corteo is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Corteo is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Corteo.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef CORTEOINDEX_H
#define CORTEOINDEX_H

// Global variables: number of errors during index fuction calls (in work if INDEX_BOUND_CHECKING is defined)
unsigned int EminErr, EmaxErr, SminErr, SmaxErr, DminErr, DmaxErr;

#define INDEX_BOUND_CHECKING  // Perform bound cheking during index evaluation (whole program 10-15% slower).
                                // No check may result out-of-bound matrix access: crash or wrong value,

// matrix index calculation parameters
#define MINE  0.0000019073486328125f  //  minimum reduced energy = 2^(-19)
#define MAXE  2097152.f // maximum reduced energy = 2^21
//#define DIME  640    // matrix dimension: (21-(-19))*2^4 (4 mantissa bits: *2^4)
//#define BIASE 1728   // (127-19)*2^4: exponent bias correction while performing division by 2^(-19)
//#define SHIFTE 19    // keep 4 of the 23 mantissa bits (23-4=19)
#define DIME  2560   // matrix dimension: (21-(-19))*2^6 (8 mantissa bits: *2^6)
#define BIASE 6912   // (127-19)*2^6: exponent bias correction while performing division by 2^(-19)
#define SHIFTE 17    // keep 6 of the 23 mantissa bits (23-6=17)

#define MINS  0.00000001490116119384765625f // mimimum reduced impact parameter = 2^(-26)
#define MAXS  64.f   // maximum reduced impact parameter = 2^6
//#define DIMS  512    // (6-(-26))*2^4
//#define BIASS 1616   // (127-26)*2^4: exponent bias correction while performing division by 2^(-26)
//#define SHIFTS 19    // keep 4 of the 23 mantissa bits (23-4=19)
#define DIMS  2048   // (6-(-26))*2^6
#define BIASS 6464  // (127-26)*2^6: exponent bias correction while performing division by 2^(-26)
#define SHIFTS 17    // keep 6 of the 23 mantissa bits (23-6=17)

#define MIND  16.f         // minimum energy (eV) for stopping power tables = 2^4
#define MAXD  1073741824.f // maximum energy (eV) for stopping power tables = 2^30 ~ 1 GeV
#define DIMD  416          //  (30-4)*2^4
#define BIASD 2096         // (127+4)*2^4: exponent bias correction while performing division by 2^10
#define SHIFTD 19          // keep 4 of the 23 mantissa bits (23-4=19)

unsigned int checkType();

// functions that compute an index from a value
unsigned int Eindex(float Eval);
unsigned int Sindex(float Sval);
unsigned int Dindex(float Dval);

// functions that return the value corresponding to an index
float Eval(unsigned int index);
float Sval(unsigned int index);
float Dval(unsigned int index);

#endif
